package eu.maiora.routes

import com.fasterxml.jackson.databind.ser.Serializers.Base
import eu.maiora.model.AccountsRepositoryImpl
import eu.maiora.model.ParametriRepositoryImpl
import io.jsonwebtoken.Jwts
import io.jsonwebtoken.SignatureAlgorithm
import io.jsonwebtoken.security.Keys
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.Json
import org.slf4j.LoggerFactory
import java.util.*


fun Route.auth(accountsRepository: AccountsRepositoryImpl, parametriRepository: ParametriRepositoryImpl){

    post("/auth"){
        // Riceve il body della richiesta e lo deserializza in ReceivedResponse
        val receivedResponse = try {
            call.receive<ReceivedResponse>()
        } catch (e: Exception) {
            return@post call.respondText("Body mancante " + e.stackTraceToString(), status = HttpStatusCode.BadRequest)
        }
        val logger = LoggerFactory.getLogger("AuthRoute")
        logger.info("param: " +
                receivedResponse.param);

        // Decodifica la stringa da Base64
        val decodedBytes = Base64.getDecoder().decode(receivedResponse.param)
        val decodedString = String(decodedBytes)
        val credentials = Json.decodeFromString<Credentials>(decodedString)




        //verifica credenziali (recupero account dal database)
        val account = accountsRepository.accountByUsername(credentials.username)
        //se le credenziali sono valide, creare il JWT
        if (account != null) {
            val passwordPlain = String(
                Base64.getDecoder().decode(
                    StringBuffer(
                        String(
                            Base64.getDecoder().decode(account.password.toByteArray())
                        )
                    ).reverse().toString().toByteArray()
                )
            )
            if(passwordPlain.equals(credentials.password)){
                val parametro = parametriRepository.parametroByChiave("jwt_secret")
                if (parametro != null) {

                    val key = Keys.hmacShaKeyFor(parametro.valore.toByteArray())
                    val now = System.currentTimeMillis()
                    val expiration = now + 86400000 // Scadenza tra 1 giorno (24 ore)

                    val token = Jwts.builder()
                        .setSubject(account.username) // Soggetto del JWT
                        .setIssuedAt(Date(now)) // Data di emissione
                        .setExpiration(Date(expiration)) // Data di scadenza
                        .signWith(key, SignatureAlgorithm.HS256) // Firma con una chiave segreta
                        .compact()

                    // Risponde con la stringa decodificata
                    call.respond(HttpStatusCode.OK, token)

                }

            }
            else {
                call.respond(HttpStatusCode.Unauthorized)
            }
        }
        else {
            call.respond(HttpStatusCode.Unauthorized)
        }

    }
}

@Serializable
data class ReceivedResponse(val param: String)

@Serializable
data class Credentials(val username: String, val password: String)